/* dxpio.c */

///// INCLUDES
#include <string.h>
#include <stdlib.h>
#include <stdio.h> 
#include <process.h>
#include "dxp_common.h"
#include "dxp_client.h"
#include "dxp_commands.h"

#if defined(__cplusplus) 
#define IMPORTED extern "C" __declspec( dllimport )
#else 
#define IMPORTED __declspec( dllimport )
#endif

///// LOCAL DEFINES
#define VER			"v1.02.007"			// version of this program
#define DXP_VER		"v1.40"				// version of the DxP protocol this program supports

///// GLOBALS
const char *error_text[] = {"Success", "Unknown command class", "Unknown command descriptor", "Invalid parameter", "WSA startup error", \
							"Socket error", "Connect fail", "Send error", "Receive error"};

///// STATIC GLOBALS (local to file)


///// FUNCTION DEFINITIONS
IMPORTED int DxP(char *, int, char *, char *, int, int, int, void *, int, void *, int *);


void usage(void)
{
	printf("\nDxP I/O Control Utility %s\n\n", VER);
	printf("Usage:\n");
	printf("  -a<ipaddress> - required - xxx.xxx.xxx.xxx\n");
	printf("  -n<ipport>    - optional - default is 9100\n");
	printf("  -u<user>      - required on supported devices\n");
	printf("  -p<password>  - required on supported devices\n");
	printf("  -c<0-8>       - optional - default is 1, 0 is all8 \n");
	printf("  -s<status>    - required (unless query) - 0 or 1\n");
	printf("  -t<time>      - optional - cycle time 1 - 99 sec.\n");
	printf("  -q<i|o>       - query (i)nput or (o)utput channel -c\n");
	printf("  -v            - version information \n");
	exit(8);
} 

/* main */
int main(int argc, char *argv[]) 
{
	// this function implements the DxP utility command line program
	char ipAddress[20] = " ";			// ip address from command line -a
	int ipPort = 9100;					// 9100 is the default DxP Port -n
	char aok = 0, sok = 0;				// boolean vars
	char cmndClass = eCmnd_io;            // default command class is eCmnd_io (0x03)
	char cmndDesc  = eCmndIoDesc_changeRelay;	// default command descriptior is eCmndIoDesc_changeRelay (0x01)
	int retval;
	tDxp dxpData;						// DxP structure to hold variables
	
	// set some defaults
	strcpy(ipAddress, "0.0.0.0");		// default IP address is 0.0.0.0 (an invalid IP address)
	ipPort = 9100;						// default IP port is 9100
	cmndClass = eCmnd_io;					// default command class is eCmnd_io (0x03)
	
	// zero out DxP structure
	memset(&dxpData, 0, sizeof(tDxp));
	
	// defaul param is zero (for now)
	dxpData.param = 0;
	
	// set default ipPort
	dxpData.ipPort = ipPort;
	
	// default sel = 1
	dxpData.sel = 1;
	
	// scan command line arguments
	while((argc > 1) && (argv[1][0] == '-'))
	{
		switch(argv[1][1])
		{
			case 'h':		// help
				usage();
				break;
				
			case 'x':		// command class
				cmndClass = atoi(&argv[1][2]);
				if((cmndClass < 1) && (cmndClass > MAX_COMMAND_CLASSES))
					cmndClass = eCmnd_io;	// default command class
				break;

			case 'a':		// IP address
				aok = 1;
				strcpy(dxpData.ipAddress, &argv[1][2]);		// look at atoi(argv[i]+2)
				break;
				
			case 'n':		// port number
				dxpData.ipPort = atoi(&argv[1][2]);
				break;
				
			case 'u':		// username
				if(strlen(&argv[1][2]) <= MAX_USERNAME_SIZE) {
					strcpy(dxpData.username, &argv[1][2]);
				}
				else {
					printf("User name must be %d characters or less\n", MAX_USERNAME_SIZE);
					exit(1);
				}
				break;

			case 'p':		// password
				if(strlen(&argv[1][2]) <= MAX_PASSWORD_SIZE) {
					strcpy(dxpData.password, &argv[1][2]);
				}
				else {
					printf("Password must be %d characters or less\n", MAX_PASSWORD_SIZE);
					exit(1);
				}
				break;

			case 's':		// state control
				sok = 1;
				switch(argv[1][2])
				{
					case '0':
						dxpData.state = OUTPUT_OPEN;
						cmndDesc  = eCmndIoDesc_changeRelay;
						break;
						
					case '1':
						dxpData.state = OUTPUT_CLOSED;
						cmndDesc  = eCmndIoDesc_changeRelay;
						break;
					
					default:
						printf("State Error, must be 0 or 1\n");
						exit(1);
				}  // end switch status
				break;
				
			case 'q':		// do query
				sok = 1;
				switch(argv[1][2])
				{
					case 'i':
						cmndDesc = eCmndIoDesc_getInputs;
						break;
						
					case 'o':
						cmndDesc = eCmndIoDesc_getRelays;
						break;
						
					default:
						printf("Query Error, must be i or o");
						exit(1);
				}// end switch
				break;

			case 't':		// pulse command
				dxpData.time = atoi(&argv[1][2]);
				if((dxpData.time >= MIN_CYCLE_TIME) && (dxpData.time <= MAX_CYCLE_TIME)) {
					cmndDesc = eCmndIoDesc_pulseRelay;
				}
				else {
					printf("Cycle time range is %d to %d\n", MIN_CYCLE_TIME, MAX_CYCLE_TIME);
					exit(1);
				}
				break;		

			case 'c':		// channel select
				dxpData.channel = atoi(&argv[1][2]);
				if(dxpData.channel == 0) {
					dxpData.sel = 0;
				}
				break;
				
			case 'v':		// version
				printf("DxP I/O Control Utility %s\n", VER);
				printf("DxP Protocol %s\n", DXP_VER);
				exit(0);
				break;

			default:		// any other argument not defined
				printf("Wrong Argument: %s\n", argv[1]);
				usage();

		} // end switch

		++argv;
		--argc;
	}  //end while

	// check for -a and -s (or -q) parameters
	if(!aok || !sok){
		printf(" -a and ( -s or -q ) parameters required\n");
		usage();
	}
	
	// done processing command line parameters, but now
	// we need to check to see if the user is trying to
	// control all of the relays, if so then we need to
	// change the command descriptor to eCmndIoDesc_changeRelays
	if((dxpData.channel == 0) && (cmndDesc == eCmndIoDesc_changeRelay)) {
		// yep, we need to change the command descriptor
		cmndDesc = eCmndIoDesc_changeRelays;
	}
	
	// at this point we have a valid IP address and status or query command
	// and possible a username/password pair
	
	// process the DxP command
	retval = processDxpCommand(cmndClass, cmndDesc, &dxpData);

	return 0; 	// same as exit(0)
}
